import 'package:flutter/material.dart';
import 'dart:ui';

enum NiceDialogType
{
  NONE,
  INFO,
  WARNING,
  ERROR,
  MULTIPLE_OPTIONS
}

class NiceDialog extends StatelessWidget{

  NiceDialogType type;

  Color typeInfoColor;
  Color typeWarningColor;
  Color typeErrorColor;
  Color typeMultipleOptionsColor;

  Color? backgroundColor;
  double? elevation;
  ShapeBorder? shape;

  Widget? header;
  Color? headerColor;
  Widget? headerIcon;
  IconData? headerIconData;
  double? headerIconSize;
  Color headerIconColor;
  double? headerPadding;
  double? borderRadius;

  Widget? title;
  String? titleText;
  Color? titleColor;

  Widget? content;
  String? message;
  Color? messageColor;

  Widget? btnOk;
  String? btnOkText;
  Color? btnOkColor;
  VoidCallback? btnOkAction;

  Widget? btnCancel;
  String? btnCancelText;
  Color? btnCancelColor;
  VoidCallback? btnCancelAction;

  Widget? btnNeutral;
  String? btnNeutralText;
  Color? btnNeutralColor;
  VoidCallback? btnNeutralAction;


  NiceDialog({
    this.type = NiceDialogType.NONE,
    this.typeInfoColor = Colors.blue,
    this.typeWarningColor = Colors.orange,
    this.typeErrorColor = const Color(0xFFC62828),
    this.typeMultipleOptionsColor = Colors.green,
    this.backgroundColor,
    this.elevation,
    this.shape,
    this.header,
    this.headerColor,
    this.headerIcon,
    this.headerIconData,
    this.headerIconSize,
    this.headerIconColor = Colors.white,
    this.headerPadding,
    this.borderRadius,
    this.title,
    this.titleText,
    this.titleColor,
    this.content,
    this.message,
    this.messageColor,
    this.btnOk,
    this.btnOkText,
    this.btnOkColor,
    this.btnOkAction,
    this.btnCancel,
    this.btnCancelText,
    this.btnCancelColor,
    this.btnCancelAction,
    this.btnNeutral,
    this.btnNeutralText,
    this.btnNeutralColor,
    this.btnNeutralAction,
  }){
    borderRadius ??= 16;
    headerPadding ??= 8;
    headerIconSize ??= 48;

    shape ??= RoundedRectangleBorder(
        borderRadius: BorderRadius.all(Radius.circular(borderRadius!))
    );

    headerIconData ??= (getDialogTypeIconData() ?? Icons.info);
    headerColor ??= (getDialogTypeColor() ?? Colors.blue);
    headerIcon ??= Icon(headerIconData, color: headerIconColor, size: headerIconSize,);
  }

  Color? getDialogTypeColor() {
    switch(type){
      case NiceDialogType.INFO: return typeInfoColor;
      case NiceDialogType.WARNING: return typeWarningColor;
      case NiceDialogType.ERROR: return typeErrorColor;
      case NiceDialogType.MULTIPLE_OPTIONS: return typeMultipleOptionsColor;
      case NiceDialogType.NONE:
      default:
        return null;
    }
  }

  IconData? getDialogTypeIconData() {
    switch(type){
      case NiceDialogType.INFO: return Icons.info;
      case NiceDialogType.WARNING: return Icons.warning;
      case NiceDialogType.ERROR: return Icons.error;
      case NiceDialogType.MULTIPLE_OPTIONS: return Icons.list;
      case NiceDialogType.NONE:
      default:
        return null;
    }
  }

  void checkParams(BuildContext context){

    titleColor ??= Theme.of(context).textTheme.headline6!.color;
    if(title == null && titleText != null)
      title = Text(titleText!, textAlign: TextAlign.center,
        style: TextStyle(fontSize: 18, color: titleColor, fontWeight: FontWeight.bold),);

    messageColor ??= Theme.of(context).textTheme.subtitle2!.color;
    if(content == null && message != null)
      content = Text(message!, textAlign: TextAlign.center,
        style: TextStyle(color: messageColor),);

    backgroundColor ??= DialogTheme.of(context).backgroundColor;
    elevation ??= DialogTheme.of(context).elevation;
    Color accentColor = Theme.of(context).accentColor;
    btnOkColor ??= accentColor;
    btnCancelColor ??= accentColor;
    btnNeutralColor ??= accentColor;
  }

  Widget? buildHeader(){
    if(header != null) return header;
    return Container(
      alignment: Alignment.center,
      padding: EdgeInsets.all(headerPadding!),
      decoration: ShapeDecoration(
        color: headerColor,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.only(
              topLeft: Radius.circular(borderRadius!),
              topRight: Radius.circular(borderRadius!))
        )
      ),
      child: headerIcon,
    );
  }

  Widget buildTitle(){
    return Container(
      margin: EdgeInsets.only(top: 10),
      child: title,
    );
  }

  Widget buildContent(){
    return Container(
      margin: EdgeInsets.only(top: 10),
      child: content,
    );
  }

  Widget? buildButtons(BuildContext context){

    if(btnNeutral == null && btnNeutralText != null)
      btnNeutral = TextButton(
        style: ButtonStyle(
          foregroundColor: MaterialStateProperty.all<Color>(btnNeutralColor ?? Theme.of(context).accentColor),
        ),
        child: Text(btnNeutralText!),
        onPressed: btnNeutralAction ?? () => Navigator.of(context).pop(),
      );
    if(btnCancel == null && btnCancelText != null)
      btnCancel = TextButton(
        style: ButtonStyle(
          foregroundColor: MaterialStateProperty.all<Color>(btnCancelColor ?? Theme.of(context).accentColor),
        ),
        child: Text(btnCancelText!),
        onPressed: btnCancelAction ?? () => Navigator.of(context).pop(),
      );
    if(btnOk == null && btnOkText != null)
      btnOk = TextButton(
        style: ButtonStyle(
          foregroundColor: MaterialStateProperty.all<Color>(btnOkColor ?? Theme.of(context).accentColor),
        ),
        child: Text(btnOkText!),
        onPressed: btnOkAction ?? () => Navigator.of(context).pop(),
      );
    return (btnNeutral != null || btnCancel != null || btnOk != null) ? Wrap(
      alignment: WrapAlignment.center,
      spacing: 8,
      children: <Widget>[
        if(btnNeutral != null) btnNeutral!,
        if(btnCancel != null) btnCancel!,
        if(btnOk != null) btnOk!,
      ],
    ) : null;
  }

  @override
  Widget build(BuildContext context) {
    checkParams(context);
    final buttons = buildButtons(context);
    return Dialog(
      backgroundColor: backgroundColor,
      elevation: elevation,
      shape: shape,
      child: Container(
        width: MediaQueryData.fromWindow(window).size.shortestSide * 60 / 100,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            buildHeader()!,
            Flexible(
              child: Scrollbar(
                child: SingleChildScrollView(
                  child: Column(
                    mainAxisSize: MainAxisSize.min,
                    children: <Widget>[
                      Padding(
                        padding: EdgeInsets.symmetric(horizontal: 16),
                        child: buildTitle(),
                      ),
                      Padding(
                        padding: EdgeInsets.symmetric(horizontal: 16),
                        child: buildContent(),
                      ),
                    ],
                  ),
                )
              ),
            ),
            if(buttons != null)
              Padding(
                padding: EdgeInsets.only(top: 16),
                child: buttons,
              )
          ],
        ),
      ),
    );
  }
}