
import 'dart:async';
import 'package:adeptviews/src/text/timer/digit.dart';
import 'package:adeptviews/src/text/timer/slide_direction.dart';
import 'package:flutter/material.dart';
import 'dart:math' as math;

class SlideCountDownClock extends StatefulWidget {
  final Duration duration;
  final TextStyle textStyle;
  final TextStyle? separatorTextStyle;
  final String separator;
  final BoxDecoration? decoration;
  final SlideDirection slideDirection;
  final VoidCallback? onDone;
  final EdgeInsets padding;
  final bool tightLabel;
  final bool shouldShowDays;

  SlideCountDownClock({
    Key? key,
    required this.duration,
    this.textStyle: const TextStyle(
      fontSize: 30,
      color: Colors.black,
    ),
    this.separatorTextStyle,
    this.decoration,
    this.tightLabel = false,
    this.separator = "",
    this.slideDirection = SlideDirection.DOWN,
    this.onDone,
    this.shouldShowDays= false,
    this.padding = EdgeInsets.zero,
  }) : super(key: key);

  @override
  _SlideCountDownClockState createState() =>
      _SlideCountDownClockState(duration, shouldShowDays);
}

class _SlideCountDownClockState extends State<SlideCountDownClock> {

  Duration timeLeft;
  bool shouldShowDays;
  StreamSubscription? onDoneStreamSubscription;
  late Stream<DateTime> timeStream;

  _SlideCountDownClockState(this.timeLeft, this.shouldShowDays) {
    if (timeLeft.inHours > 99) {
      shouldShowDays = true;
    }
  }

  @override
  void initState() {
    super.initState();
    _init();
  }

  void _init() {
    final frequency = Duration(seconds: 1);

    if(widget.onDone != null) {
      final onDoneStream = Stream<DateTime?>.periodic(frequency, (_) {
        timeLeft -= Duration(seconds: 1);
        if (timeLeft.inSeconds == 0) {
          onDoneStreamSubscription?.cancel();
          Future.delayed(Duration(seconds: 1), () {
            widget.onDone?.call();
          });
        }
        return null;
      });
      onDoneStreamSubscription = onDoneStream.listen(null);
    }

    timeStream = Stream<DateTime>.periodic(frequency, null)
        .take(timeLeft.inSeconds).asBroadcastStream();
  }

  @override
  Widget build(BuildContext context) {
    Widget dayDigits;
    if (timeLeft.inDays > 99) {
      List<DigitFunction> digits = [];
      for (int i = timeLeft.inDays.toString().length - 1; i >= 0; i--) {
        digits.add((DateTime? time) =>
            ((timeLeft.inDays) ~/ math.pow(10, i) % math.pow(10, 1)).toInt());
      }
      dayDigits = _buildDigitForLargeNumber(
          timeStream, digits, DateTime.now(), 'daysHundreds');
    } else {
      dayDigits = _buildDigit(
        timeStream,
            (DateTime? time) => (timeLeft.inDays) ~/ 10,
            (DateTime? time) => (timeLeft.inDays) % 10,
        DateTime.now(),
        "Days",
      );
    }

    return Row(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        (shouldShowDays) ? dayDigits : SizedBox(),
        (shouldShowDays) ? _buildSpace() : SizedBox(),
        (widget.separator.isNotEmpty && shouldShowDays)
            ? _buildSeparator()
            : SizedBox(),
        _buildDigit(
          timeStream,
              (DateTime? time) => (timeLeft.inHours % 24) ~/ 10,
              (DateTime? time) => (timeLeft.inHours % 24) % 10,
          DateTime.now(),
          "Hours",
        ),
        _buildSpace(),
        (widget.separator.isNotEmpty) ? _buildSeparator() : SizedBox(),
        _buildSpace(),
        _buildDigit(
          timeStream,
              (DateTime? time) => (timeLeft.inMinutes % 60) ~/ 10,
              (DateTime? time) => (timeLeft.inMinutes % 60) % 10,
          DateTime.now(),
          "minutes",
        ),
        _buildSpace(),
        (widget.separator.isNotEmpty) ? _buildSeparator() : SizedBox(),
        _buildSpace(),
        _buildDigit(
          timeStream,
              (DateTime? time) => (timeLeft.inSeconds % 60) ~/ 10,
              (DateTime? time) => (timeLeft.inSeconds % 60) % 10,
          DateTime.now(),
          "seconds",
        )
      ],
    );
  }

  Widget _buildSpace() {
    return SizedBox(width: 3);
  }

  Widget _buildSeparator() {
    return Text(
      widget.separator,
      style: widget.separatorTextStyle ?? widget.textStyle,
    );
  }

  Widget _buildDigitForLargeNumber(
      Stream<DateTime> timeStream,
      List<DigitFunction> digits,
      DateTime startTime,
      String id,
      ) {
    String timeLeftString = timeLeft.inDays.toString();
    List<Widget> rows = [];
    for (int i = 0; i < timeLeftString.toString().length; i++) {
      rows.add(
        Container(
          decoration: widget.decoration,
          padding:
          widget.tightLabel ? EdgeInsets.only(left: 3) : EdgeInsets.zero,
          child: Digit(
            padding: widget.padding,
            itemStream: timeStream.map<int>(digits[i]),
            initValue: digits[i](startTime),
            decoration: widget.decoration,
            slideDirection: widget.slideDirection,
            textStyle: widget.textStyle,
          ),
        ),
      );
    }

    return Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: rows,
        ),
      ],
    );
  }

  Widget _buildDigit(
      Stream<DateTime> timeStream,
      DigitFunction tensDigit,
      DigitFunction onesDigit,
      DateTime startTime,
      String id,
      ) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Container(
              decoration: widget.decoration,
              padding: widget.tightLabel
                  ? EdgeInsets.only(left: 3)
                  : EdgeInsets.zero,
              child: Digit(
                padding: widget.padding,
                itemStream: timeStream.map<int>(tensDigit),
                initValue: tensDigit(startTime),
                decoration: widget.decoration,
                slideDirection: widget.slideDirection,
                textStyle: widget.textStyle,
              ),
            ),
            Container(
              decoration: widget.decoration,
              padding: widget.tightLabel
                  ? EdgeInsets.only(right: 3)
                  : EdgeInsets.zero,
              child: Digit(
                padding: widget.padding,
                itemStream: timeStream.map<int>(onesDigit),
                initValue: onesDigit(startTime),
                decoration: widget.decoration,
                slideDirection: widget.slideDirection,
                textStyle: widget.textStyle,
              ),
            ),
          ],
        ),
      ],
    );
  }

  @override
  void dispose() {
    onDoneStreamSubscription?.cancel();
    super.dispose();
  }
}
